@@ -6,6 +6,7 @@ Created on 2022-09-02
 import asyncio
 import fnmatch
 import inspect
+import importlib
 import json
 import logging
 import os
@@ -21,7 +22,7 @@ from sys import platform
 from multiprocessing import Process
 from threading import Thread
 from starlette.applications import Starlette
-from starlette.routing import Route,Match
+from starlette.routing import Route,request_response
 from starlette.endpoints import HTTPEndpoint
 from starlette.responses import HTMLResponse, JSONResponse,PlainTextResponse, Response
 from starlette.templating import Jinja2Templates
@@ -327,9 +328,9 @@ class JustpyApp(Starlette):
         assert issubclass(
             page_type, WebPage
         ), f"Function did not return a web page but a {page_type.__name__}"
-        assert (
-            len(load_page) > 0 or load_page.html
-        ), "\u001b[47;1m\033[93mWeb page is empty, add components\033[0m"
+        if len(load_page) == 0 and not load_page.html:
+            error_html="""<span style="color:red">Web page is empty - you might want to add components</span>"""
+            return HTMLResponse(error_html, 500)
         page_options = {
             "reload_interval": load_page.reload_interval,
             "body_style": load_page.body_style,
@@ -600,24 +601,34 @@ class JustpyServer:
         url = f"http://{self.host}:{self.port}{path}"
         return url
 
-
 class JustpyDemoApp:
     """
     a justpy application
     """
 
-    def __init__(self, pymodule_file, wp: typing.Callable = None, **kwargs):
+    def __init__(self,examples_dir:str,pymodule_file:str, wpfunc: typing.Callable = None, debug:bool=False,**kwargs):
         """
         Args:
+            examples_dir(str): the root directory for the examples
             pymodule_file(str): the python module path where the app resides
-            wp(Callable): the webpage function to call
+            wpfunc(Callable): the webpage function to call
+            debug(bool): if True switch on debug mode
             **kwargs: further keyword arguments to pass to the webpage function
         """
+        self.examples_dir=examples_dir
         self.pymodule_file = pymodule_file
         with open(self.pymodule_file, "r") as sourcefile:
             self.source = sourcefile.read()
             self.check_demo()
-        self.wp = wp
+        source_file=pymodule_file
+        self.source_file=source_file.replace(self.examples_dir+"/","")
+        self.source_link=f"""<a href="https://github.com/justpy-org/justpy/blob/master/examples/{self.source_file}">{self.source_file}</a>"""
+        self.video_url=None
+        self.try_it_url=None
+        self.name=os.path.basename(self.source_file).replace(".py","")
+        self.wpfunc = wpfunc
+        self.debug=debug
+        self.status=""
         self.kwargs = kwargs
 
     def check_demo(self):
@@ -626,12 +637,12 @@ class JustpyDemoApp:
         """
         self.is_demo = False
         if "Demo(" or "Demo (" in self.source:
-            endpoint_match = re.search(
+            func_match = re.search(
                 """Demo[ ]?[(]["'](.*)["'],\s*(.*?)(,.*)*[)]""", self.source
             )
-            if endpoint_match:
-                self.description = endpoint_match.group(1)
-                self.endpoint = endpoint_match.group(2)
+            if func_match:
+                self.description = func_match.group(1)
+                self.wpfunc_name = func_match.group(2)
                 # jenkins could have /var/lib/jenkins/jobs/justpy/workspace/examples/charts_tutorial/pandas/women_majors2.py is not a demo
                 module_match = re.search(
                     "justpy/(workspace/)?(examples/.*)[.]py", self.pymodule_file
@@ -639,20 +650,29 @@ class JustpyDemoApp:
                 if module_match:
                     self.pymodule = module_match.group(2)
                     self.pymodule = self.pymodule.replace("/", ".")
-                    self.is_demo = not "lambda" in self.endpoint
+                    self.is_demo = not "lambda" in self.wpfunc_name
                 return
 
-    async def start(self, server: JustpyServer):
+    def mount(self, app:JustpyApp):
         """
-        Start me on the given server
+        mount me on the given app
 
         Args:
-            server(JustpyServer): the server to start
+            app(JustpyApp): the app to mount me on
         """
-        if self.wp is None:
-            raise Exception(f"can't start {self.pymodule_file} -wp/endoint is None")
-        return await server.start(self.wp)
+        demo_module = importlib.import_module(self.pymodule)
+        self.wpfunc = getattr(demo_module, self.wpfunc_name)
+        if self.wpfunc is None:
+            raise Exception(f"can't start {self.pymodule_file} -wpfunc/endoint is None")
+        # https://fastapi.tiangolo.com/advanced/sub-applications/
+        name=self.wpfunc.__name__
+        endpoint=app.response(self.wpfunc)
+        # https://github.com/encode/starlette/issues/734
+        mount_app=request_response(endpoint)
+        if self.debug:
+            print(f"mounting  {str(self)}")
+        app.mount(f"/{name}",mount_app,name)
 
     def __str__(self):
-        text = f"{self.pymodule}:{self.endpoint}:{self.description}"
+        text = f"{self.pymodule}:{self.wpfunc_name}:{self.description}"
         return text
